class InvalidTemplateError(StandardError):
    pass



class InvalidColourError(StandardError):
    pass



class InvalidOpacityError(StandardError):
    pass



class InvalidHueError(StandardError):
    pass
